package logger

import (
	config2 "dc_golang_server_1/config"
	"github.com/gin-gonic/gin"
	"github.com/natefinch/lumberjack"
	"go.uber.org/zap"
	"go.uber.org/zap/zapcore"
	"os"
	"time"
)

var Log *zap.Logger

func InitLogger() {
	config := zapcore.EncoderConfig{
		MessageKey:   "msg",                       //结构化（json）输出：msg的key
		LevelKey:     "level",                     //结构化（json）输出：日志级别的key（INFO，WARN，ERROR等）
		TimeKey:      "ts",                        //结构化（json）输出：时间的key（INFO，WARN，ERROR等）
		CallerKey:    "file",                      //结构化（json）输出：打印日志的文件对应的Key
		EncodeLevel:  zapcore.CapitalLevelEncoder, //将日志级别转换成大写（INFO，WARN，ERROR等）
		EncodeCaller: zapcore.ShortCallerEncoder,  //采用短文件路径编码输出（test/main.go:14 ）
		EncodeTime: func(t time.Time, enc zapcore.PrimitiveArrayEncoder) {
			enc.AppendString(t.Format("2006-01-02 15:04:05.000"))
		}, //输出的时间格式
		//EncodeDuration: func(d time.Duration, enc zapcore.PrimitiveArrayEncoder) {
		//	enc.AppendInt64(int64(d) / 1000000)
		//},//
	}

	//debug info
	infoLevel := zap.LevelEnablerFunc(func(lvl zapcore.Level) bool {
		return lvl < zapcore.WarnLevel && lvl >= config2.LogLevel
	})

	//warn
	warnLevel := zap.LevelEnablerFunc(func(lvl zapcore.Level) bool {
		return lvl == zapcore.WarnLevel && lvl >= config2.LogLevel
	})

	//error DPanic Panci Fatal
	errorLevel := zap.LevelEnablerFunc(func(lvl zapcore.Level) bool {
		return lvl > zapcore.WarnLevel && lvl >= config2.LogLevel
	})

	infoWriter := &lumberjack.Logger{
		Filename:   config2.LogPath + "info.log",
		MaxSize:    config2.LogFileMaxSize,    //最大M数，超过则切割
		MaxBackups: config2.LogInfoMaxFileNum, //最大文件保留数，超过就删除最老的日志文件
		MaxAge:     config2.LogInfoMaxFileDay, //保存30天
		Compress:   false,                     //是否压缩
	}
	warnWriter := &lumberjack.Logger{
		Filename:   config2.LogPath + "warn.log",
		MaxSize:    config2.LogFileMaxSize,    //最大M数，超过则切割
		MaxBackups: config2.LogWarnMaxFileNum, //最大文件保留数，超过就删除最老的日志文件
		MaxAge:     config2.LogWarnMaxFileDay, //保存30天
		Compress:   false,                     //是否压缩
	}
	errorWriter := &lumberjack.Logger{
		Filename:   config2.LogPath + "error.log",
		MaxSize:    config2.LogFileMaxSize,     //最大M数，超过则切割
		MaxBackups: config2.LogErrorMaxFileNum, //最大文件保留数，超过就删除最老的日志文件
		MaxAge:     config2.LogErrorMaxFileDay, //保存30天
		Compress:   false,                      //是否压缩
	}

	core := zapcore.NewTee(
		zapcore.NewCore(zapcore.NewJSONEncoder(config), zapcore.AddSync(infoWriter), infoLevel),   // zapcore.NewConsoleEncoder(encoder)
		zapcore.NewCore(zapcore.NewJSONEncoder(config), zapcore.AddSync(warnWriter), warnLevel),   //
		zapcore.NewCore(zapcore.NewJSONEncoder(config), zapcore.AddSync(errorWriter), errorLevel), //
		//zapcore.NewCore(zapcore.NewJSONEncoder(encoder), zapcore.NewMultiWriteSyncer(zapcore.AddSync(os.Stdout)), logLevel),//同时将日志输出到控制台，NewJSONEncoder 是结构化输出
	)
	Log = zap.New(core, zap.AddStacktrace(zap.ErrorLevel))
}

func HttpGinLog() gin.HandlerFunc { //logger *log.Log
	return func(c *gin.Context) {
		startTime := time.Now().UnixNano()
		c.Next()
		// stopTime := time.Since(startTime)
		// spendTime := stopTime.Nanoseconds()/1000000///fmt.Sprintf("%d ms", (stopTime.Nanoseconds()+500000)/1000000)
		spendTime := time.Now().UnixNano() - startTime
		hostName, err := os.Hostname()
		if err != nil {
			hostName = "unknown"
		}
		statusCode := c.Writer.Status()
		clientIp := c.ClientIP()
		userAgent := c.Request.UserAgent()
		dataSize := c.Writer.Size()
		if dataSize < 0 {
			dataSize = 0
		}
		method := c.Request.Method
		path := c.Request.RequestURI

		//c.Request.Body.Read()
		//body := c.Request.Body
		//para := c.Params

		//entry := Log.WithFields(log.Fields{
		//	"HostName":  hostName,
		//	"Status":    statusCode,
		//	"SpendNanoS": spendTime,
		//	"Ip":        clientIp,
		//	"Method":    method,
		//	"Path":      path,
		//	"DataSize":  dataSize,
		//	"Agent":     userAgent,
		//})

		if len(c.Errors) > 0 {
			//entry.Error(c.Errors.ByType(gin.ErrorTypePrivate).String())
			Log.Error(c.Errors.ByType(gin.ErrorTypePrivate).String(),
				zap.String("HostName", hostName),
				zap.Int("Status", statusCode),
				zap.Int64("SpendNanoS", spendTime),
				zap.String("Ip", clientIp),
				zap.String("Method", method),
				zap.String("Path", path),
				zap.Int("DataSize", dataSize),
				zap.String("Agent", userAgent))
		}
		if statusCode >= 500 {
			Log.Error("",
				zap.String("HostName", hostName),
				zap.Int("Status", statusCode),
				zap.Int64("SpendNanoS", spendTime),
				zap.String("Ip", clientIp),
				zap.String("Method", method),
				zap.String("Path", path),
				zap.Int("DataSize", dataSize),
				zap.String("Agent", userAgent))
		} else if statusCode >= 400 {
			Log.Warn("",
				zap.String("HostName", hostName),
				zap.Int("Status", statusCode),
				zap.Int64("SpendNanoS", spendTime),
				zap.String("Ip", clientIp),
				zap.String("Method", method),
				zap.String("Path", path),
				zap.Int("DataSize", dataSize),
				zap.String("Agent", userAgent))
		} else {
			Log.Info("",
				zap.String("HostName", hostName),
				zap.Int("Status", statusCode),
				zap.Int64("SpendNanoS", spendTime),
				zap.String("Ip", clientIp),
				zap.String("Method", method),
				zap.String("Path", path),
				zap.Int("DataSize", dataSize),
				zap.String("Agent", userAgent))
		}
	}
}

//import (
//	"github.com/gin-gonic/gin"
//	rotatelogs "github.com/lestrrat-go/file-rotatelogs"
//	"github.com/rifflock/lfshook"
//	log "github.com/sirupsen/logrus"
//	"jeff_workstation_1/util"
//	"os"
//	"time"
//)
//
//func InitLogger(filePath string){
//
//	// log.SetOutput(colorable.NewColorableStdout())
//
//	// log.SetOutput(f)
//	log.SetLevel(util.LogLevel)
//
//	TraceLogWriter,_ := rotatelogs.New(
//		filePath+"Trace_%Y%m%d%H.log",
//		rotatelogs.WithMaxAge(util.LogTraceWithMaxAge),
//		rotatelogs.WithRotationTime(util.LogTraceWithRotationTime),
//		//rotatelogs.WithLinkName(linkName),
//	)
//	DebugLogWriter,_ := rotatelogs.New(
//		filePath+"Debug_%Y%m%d%H.log",
//		rotatelogs.WithMaxAge(util.LogDebugWithMaxAge),
//		rotatelogs.WithRotationTime(util.LogDebugWithRotationTime),
//		//rotatelogs.WithLinkName(linkName),
//	)
//	infoLogWriter,_ := rotatelogs.New(
//		filePath+"Info_%Y%m%d%H.log",
//		rotatelogs.WithMaxAge(util.LogInfoWithMaxAge),
//		rotatelogs.WithRotationTime(util.LogInfoWithRotationTime),
//		//rotatelogs.WithLinkName("tcp_info.log"),
//	)
//	WarnLogWriter,_ := rotatelogs.New(
//		filePath+"Warn_%Y%m%d%H.log",
//		rotatelogs.WithMaxAge(util.LogWarnWithMaxAge),
//		rotatelogs.WithRotationTime(util.LogWarnWithRotationTime),
//		//rotatelogs.WithLinkName("tcp_warn.log"),
//	)
//	ErrorLogWriter,_ := rotatelogs.New(
//		filePath+"Error_%Y%m%d%H.log",
//		rotatelogs.WithMaxAge(util.LogErrorWithMaxAge),
//		rotatelogs.WithRotationTime(util.LogErrorWithRotationTime),
//		//rotatelogs.WithLinkName("tcp_error.log"),
//	)
//
//	writeMap := lfshook.WriterMap{
//		log.TraceLevel: TraceLogWriter,
//		log.DebugLevel: DebugLogWriter,
//		log.InfoLevel:  infoLogWriter,
//		log.WarnLevel:  WarnLogWriter,
//		log.ErrorLevel: ErrorLogWriter,
//		log.PanicLevel: ErrorLogWriter,
//		log.FatalLevel: ErrorLogWriter,
//	}
//	hook := lfshook.NewHook(writeMap,&log.JSONFormatter{
//		TimestampFormat: "2006-01-02 15:04:05.000000",
//	})
//
//	log.AddHook(hook)
//	return
//}
//
//func HttpGinLog() gin.HandlerFunc{ //logger *log.Log
//	return func(c *gin.Context) {
//		startTime := time.Now().UnixNano()
//		c.Next()
//		//stopTime := time.Since(startTime)
//		//spendTime := stopTime.Nanoseconds()/1000000///fmt.Sprintf("%d ms", (stopTime.Nanoseconds()+500000)/1000000)
//		spendTime := time.Now().UnixNano() - startTime
//		hostName, err := os.Hostname()
//		if err != nil {
//			hostName = "unknown"
//		}
//		statusCode := c.Writer.Status()
//		clientIp := c.ClientIP()
//		userAgent := c.Request.UserAgent()
//		dataSize := c.Writer.Size()
//		if dataSize < 0 {
//			dataSize = 0
//		}
//		method := c.Request.Method
//		path := c.Request.RequestURI
//
//		//c.Request.Body.Read()
//		//body := c.Request.Body
//		//para := c.Params
//
//		entry := log.WithFields(log.Fields{
//			"HostName":  hostName,
//			"Status":    statusCode,
//			"SpendNanoS": spendTime,
//			"Ip":        clientIp,
//			"Method":    method,
//			"Path":      path,
//			"DataSize":  dataSize,
//			"Agent":     userAgent,
//		})
//
//		if len(c.Errors) > 0 {
//			entry.Error(c.Errors.ByType(gin.ErrorTypePrivate).String())
//		}
//		if statusCode >= 500 {
//			entry.Error()
//		} else if statusCode >= 400 {
//			entry.Warn()
//		} else {
//			entry.Info()
//		}
//	}
//}
